unit FileDeleter;

interface
uses
  classes, SysUtils;

type
  TFileDeleter = class
  public
    constructor Create();
    function addFileNameMask(mask : string) : integer;
    function addDirectory(dir : string) : integer;
    procedure setRecursiveDirs(doRecursive : boolean);
    function delFileNameMask(mask : string) : integer;
    function delDirectory(dir : string) : integer;
    procedure setPreviewMode(doPreview : boolean);

    procedure delFiles(deletedFilesList : TStrings); overload;

    function clearAllFileNameMasks : integer;
    function clearAllDirectories : integer;
    procedure resetToDefault;


//get methods
    function getPreviewMode: boolean;
    function getRecursiveDirs : boolean;
    procedure getDirectories(dirs : TStrings);
    procedure getFileNameMasks(masks : TStrings);


//methods to deal with saved settings
    function setParamsFileName(fileName : string) : integer;
    function loadParamsFromFile : integer;
    function saveParamsToFile : integer;
{
    function saveParamsToFile : integer;
}

//TODO : to be moved to private!!
    procedure getSubDirs(startDir : string; subDirList : TStrings);
  public
    dirList, maskList : TStringList;
  private
    previewMode, recSubDirs : boolean;
    paramsFileName : string;


    procedure localDelFiles(dir, mask : string; deletedFilesList : TStrings);
    procedure localDelFilesBase(dir, mask : string; deletedFilesList : TStrings);
    function isValidMask(mask : string) : boolean;

  end;
implementation

uses
  FileCtrl;


  const
    paramsDirsListTitle = '[Directories]';
    paramsMasksListTitle = '[FileNameMasks]';
    paramsOptionsTitle = '[Options]';
    paramsRecursiveDirs = 'RecursiveDirs';

    paramsFirstLine = 'This is a generated file - Please DO NOT Edit!';

///////public

function TFileDeleter.addDirectory(dir: string): integer;
begin
  if not DirectoryExists(dir) then
  begin
    Result := 1;
    exit;
  end;
  if dirList.IndexOf(dir) > -1 then
  begin
    Result := 2;
    exit;
  end;

  dirList.Add(dir);
  Result := 0;
end;

function TFileDeleter.addFileNameMask(mask: string): integer;
begin
  Result := 1;
  if isValidMask(mask) then
  begin
    maskList.Add(mask);
    Result := 0;
  end;
end;

function TFileDeleter.delDirectory(dir: string): integer;
begin
  Result := 1;
  if dirList.IndexOf(dir) > -1 then
  begin
    dirList.Delete(dirList.IndexOf(dir));
    Result := 0;
  end;
end;

function TFileDeleter.delFileNameMask(mask: string): integer;
begin
  Result := 1;
  if maskList.IndexOf(mask) > -1 then
  begin
    maskList.Delete(maskList.IndexOf(mask));
    Result := 0;
  end;

end;

procedure TFileDeleter.setRecursiveDirs(doRecursive: boolean);
begin
  recSubDirs := doRecursive;
end;

constructor TFileDeleter.Create;
begin
  inherited;
  resetToDefault;
end;

procedure TFileDeleter.delFiles(deletedFilesList: TStrings);
var
  i, j : integer;
begin
  deletedFilesList.Clear;
  if (dirList.Count > 0) and (maskList.Count > 0) then
  for i := 0 to dirList.Count -1 do
  begin
    for j := 0 to maskList.Count -1 do
    begin
      localDelFiles(dirList[i], maskList[j], deletedFilesList);
    end;
  end;
end;


procedure TFileDeleter.setPreviewMode(doPreview: boolean);
begin
  previewMode := doPreview;
end;

///////public - end

procedure TFileDeleter.localDelFiles(dir, mask: string;
  deletedFilesList: TStrings);
var
  subDirs : TStringList;
  i : integer;
begin
  if not recSubDirs then
    localDelFilesBase(dir, mask, deletedFilesList)
  else
  begin
    localDelFilesBase(dir, mask, deletedFilesList);  //always the current directory

    subDirs := TStringList.Create;
    getSubDirs(dir, subDirs);
    for i := 0 to subDirs.Count -1 do
    begin
      localDelFiles(subDirs[i], mask, deletedFilesList);
    end;
  end;
end;

procedure TFileDeleter.getSubDirs(startDir: string; subDirList: TStrings);
var
  sr: TSearchRec;
  i, ii : integer;
begin
  try
    subDirList.Clear;
    if FindFirst(startDir + '\*.', faDirectory, sr) = 0 then //something found
    repeat
      subDirList.Add(startDir + '\' + sr.Name);
    until FindNext(sr) <> 0;

//remove . e .. directories
    i := subDirList.IndexOf(startDir + '\' + '.');
    if i > -1 then
      subDirList.Delete(i);
    ii := subDirList.IndexOf(startDir + '\' + '..');
    if ii > -1 then
      subDirList.Delete(ii);
  finally
    FindClose(sr);
  end;
end;

procedure TFileDeleter.localDelFilesBase(dir, mask: string;
  deletedFilesList: TStrings);
var
  sr: TSearchRec;
begin
  try
    if FindFirst(dir + '\' + mask, 0, sr) = 0 then //something found
    repeat
      if previewMode then
        deletedFilesList.Add(dir + '\' + sr.Name)
      else if DeleteFile(dir + '\' + sr.Name) then
        deletedFilesList.Add(dir + '\' + sr.Name)
      else
        deletedFilesList.Add('NOT DELETED: ' + dir + '\' + sr.Name);
    until FindNext(sr) <> 0;
  finally
    FindClose(sr);
  end;
end;



function TFileDeleter.isValidMask(mask: string): boolean;
begin
//TODO : isValidMask : it checks if the mask is correct.
  Result := True;
end;


function TFileDeleter.clearAllDirectories: integer;
begin
  Result := 1;
  if dirList.Count > 0 then
  begin
    dirList.Clear;
    Result := 0;
  end;
end;

function TFileDeleter.clearAllFileNameMasks: integer;
begin
  Result := 1;
  if maskList.Count > 0 then
  begin
    maskList.Clear;
    Result := 0;
  end;
end;

procedure TFileDeleter.resetToDefault;
begin
  if not Assigned(maskList) then
    maskList := TStringList.Create()
  else
    clearAllFileNameMasks;

  if not Assigned(dirList) then
    dirList := TStringList.Create()
  else
    clearAllDirectories;

  setRecursiveDirs(False);
  setPreviewMode(False);
  paramsFileName := '';
end;

function TFileDeleter.loadParamsFromFile: integer;
var
  dirs, masks, fileLines : TStringList;
  f : TextFile;
  s : string;
  i : integer;
  recDirs : integer;
begin
  if (paramsFileName <> '') and (FileExists(paramsFileName)) then
  begin
    dirs := TStringList.Create;
    masks := TStringList.Create;
    fileLines := TStringList.Create;

    //read the file into fileLines
    AssignFile(f, paramsFileName);
    Reset(f);
    while not eof(f) do
    begin
      Readln(f, s);
      fileLines.Add(s);
    end;
    CloseFile(f);

    //search for the directories list
    i := fileLines.IndexOf(paramsDirsListTitle);
    if i <> -1 then
    begin
      Inc(i);//next item
      repeat
        if DirectoryExists(fileLines.Strings[i]) then
          dirs.Add(fileLines.Strings[i])
        else
        begin
          dirs.Clear;
          break;
        end;
        Inc(i);
      until (fileLines.Count < i) OR (fileLines.Strings[i] = '') OR
        (fileLines.Strings[i] = paramsMasksListTitle);
    end;

    //search for the fileName masks list
    i := fileLines.IndexOf(paramsMasksListTitle);
    if i <> -1 then
    begin
      Inc(i);//next item
      repeat
        if isValidMask(fileLines.Strings[i]) then
          masks.Add(fileLines.Strings[i])
        else
        begin
          masks.Clear;
          break;
        end;
        Inc(i);
      until (fileLines.Count < i) OR (fileLines.Strings[i] = '') OR
        (fileLines.Strings[i] = paramsOptionsTitle);
    end;

    //search for the options list
    recDirs := 0;
    i := fileLines.IndexOfName(paramsRecursiveDirs);
    if i <> -1 then  //there is paramsRecursiveDirs option
    begin
      recDirs := StrToInt(fileLines.Values[paramsRecursiveDirs]);
    end;


    if (dirs.Count = 0) OR (masks.Count = 0) then  //wrong information provided
      Result := 1
    else
    begin
//clear the previous values
      clearAllDirectories;
      dirList.Assign(dirs);
      clearAllFileNameMasks;
      maskList.Assign(masks);
      setRecursiveDirs(recDirs = 1);
      setPreviewMode(False);
      Result := 0;  //ok
    end;
  end
  else
  begin
    Result := 1;
  end;
end;

function TFileDeleter.setParamsFileName(fileName: string): integer;
begin
  paramsFileName := fileName;
  Result := 0;
end;

procedure TFileDeleter.getDirectories(dirs: TStrings);
begin
  dirs.Assign(dirList);
end;

procedure TFileDeleter.getFileNameMasks(masks: TStrings);
begin
  masks.Assign(maskList);
end;

function TFileDeleter.getPreviewMode: boolean;
begin
  Result := previewMode;
end;

function TFileDeleter.getRecursiveDirs: boolean;
begin
  Result := recSubDirs;
end;

function TFileDeleter.saveParamsToFile: integer;
var
  fileLines : TStringList;
  i : integer;
begin
  Result := 1;
  if paramsFileName = '' then
    Exit;
  Result := 2;
  if (dirList.Count = 0) or (maskList.Count = 0) then
    Exit;

  Result := 3;

  fileLines := TStringList.Create;
  fileLines.Add(paramsFirstLine);
  fileLines.Add('Generated on: ' + DateTimeToStr(Date() + Time()));
  fileLines.Add('');
  fileLines.Add(paramsDirsListTitle);

//add the list of dirs:
  for i := 0 to dirList.Count -1 do
    fileLines.Add(dirList.Strings[i]);
  fileLines.Add('');

//add the list of masks:
  fileLines.Add(paramsMasksListTitle);
  for i := 0 to maskList.Count -1 do
    fileLines.Add(maskList.Strings[i]);
  fileLines.Add('');


    //search for the options list
  fileLines.Add(paramsOptionsTitle);
  if getRecursiveDirs then
    fileLines.Add(paramsRecursiveDirs + '=1')
  else
    fileLines.Add(paramsRecursiveDirs + '=0');
  try
    fileLines.SaveToFile(paramsFileName);
    Result := 0;
  except
  end;
end;

end.
